"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.base64ToArrayBuffer = exports.getSegmentOffset = exports.processMidiFile = exports.base64ToBinary = exports.getDurationOfNotes = exports.getBeatLength = exports.sliceAndDice = exports.getPartitioningArrayWithMax = exports.getSimplePartitioningArray = exports.shiftMidi = exports.getValidTracks = exports.MAX_BREATH_SECONDS = void 0;
const notes_processor_1 = __importDefault(require("./notes-processor"));
exports.MAX_BREATH_SECONDS = 7;
const getValidTracks = (midi) => {
    return midi.tracks.filter((track) => track.notes.length > 0 && track.channel >= 0);
};
exports.getValidTracks = getValidTracks;
const shiftMidi = (midi, amount) => {
    for (const track of midi.tracks) {
        for (const note of track.notes) {
            note.time += amount;
        }
    }
};
exports.shiftMidi = shiftMidi;
const getSimplePartitioningArray = (arr) => {
    return arr
        .map((num, index) => (index === 0 || num !== 0 ? index : 0))
        .filter((num, index) => index === 0 || num !== 0);
};
exports.getSimplePartitioningArray = getSimplePartitioningArray;
const getPartitioningArrayWithMax = (arr, max) => {
    const dividingIndicies = [];
    let sum = 0;
    for (let i = 0; i < arr.length; i++) {
        sum += arr[i];
        if (sum > max || i === 0) {
            dividingIndicies.push(i);
            sum = arr[i];
        }
    }
    return dividingIndicies;
};
exports.getPartitioningArrayWithMax = getPartitioningArrayWithMax;
const sliceAndDice = (arr, indicesToSliceAt) => {
    let ret = [];
    for (let i = 0; i < indicesToSliceAt.length; i++) {
        const partitionIndex = indicesToSliceAt[i];
        const nextPartitionIndex = indicesToSliceAt[i + 1];
        ret.push(arr.slice(partitionIndex, nextPartitionIndex));
    }
    return ret;
};
exports.sliceAndDice = sliceAndDice;
const getBeatLength = (bpm) => 1 / (bpm / 60);
exports.getBeatLength = getBeatLength;
const getDurationOfNotes = (notes) => {
    const startTime = notes[0].time;
    const lastNote = notes[notes.length - 1];
    const { time, duration } = lastNote;
    return time + duration - startTime;
};
exports.getDurationOfNotes = getDurationOfNotes;
const base64ToBinary = (base64String) => Buffer.from(base64String, "base64").toString("binary");
exports.base64ToBinary = base64ToBinary;
const processMidiFile = (midi) => {
    const beatLength = (0, exports.getBeatLength)(midi.simpleBpm);
    const shift = beatLength * 2;
    (0, exports.shiftMidi)(midi, shift);
    const tracks = (0, exports.getValidTracks)(midi);
    const segmentInfos = [];
    // segmentize
    tracks.forEach((track) => {
        const notesGroupedOnRests = notes_processor_1.default.groupNotesOnRests(track.notes);
        const divisions = notes_processor_1.default.subdivideUnderMaxBreath(notesGroupedOnRests, exports.MAX_BREATH_SECONDS);
        // clump them together
        const finalDivisions = notes_processor_1.default.clumpTogether(divisions, exports.MAX_BREATH_SECONDS);
        finalDivisions.forEach((notes) => {
            const offset = getSegmentOffset(notes[0].time, beatLength, shift);
            let lowestNote = Infinity;
            let highestNote = -Infinity;
            const finalNotes = notes.map((note) => {
                lowestNote = Math.min(lowestNote, note.midi);
                highestNote = Math.max(highestNote, note.midi);
                return {
                    midi: note.midi,
                    duration: note.duration,
                    velocity: note.velocity,
                    time: note.time - offset,
                };
            });
            segmentInfos.push({
                bpm: midi.simpleBpm,
                offset,
                notes: finalNotes,
                lowestNote,
                highestNote,
            });
        });
    });
    return segmentInfos;
};
exports.processMidiFile = processMidiFile;
function getSegmentOffset(firstNoteStart, beatLength, shift) {
    const minimumOffset = firstNoteStart - (firstNoteStart % beatLength);
    const offset = minimumOffset - shift;
    return offset;
}
exports.getSegmentOffset = getSegmentOffset;
function base64ToArrayBuffer(base64) {
    const binary_string = window.atob(base64);
    const len = binary_string.length;
    const bytes = new Uint8Array(len);
    for (let i = 0; i < len; i++) {
        bytes[i] = binary_string.charCodeAt(i);
    }
    return bytes.buffer;
}
exports.base64ToArrayBuffer = base64ToArrayBuffer;
