"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
class NotesProcessor {
}
exports.default = NotesProcessor;
NotesProcessor.getTimeDifferenceArray = (notes) => {
    const diffArray = [0];
    for (let i = 1; i < notes.length; i++) {
        const previousNote = notes[i - 1];
        const diff = notes[i].time - (previousNote.duration + previousNote.time);
        diffArray.push(diff);
    }
    return diffArray;
};
NotesProcessor.groupNotesOnRests = (notes) => {
    const diffArray = NotesProcessor.getTimeDifferenceArray(notes);
    const partitioningArray = (0, helpers_1.getSimplePartitioningArray)(diffArray);
    return (0, helpers_1.sliceAndDice)(notes, partitioningArray);
};
NotesProcessor.subdivideUnderMaxBreath = (noteGroupings, maxBreathSeconds) => {
    let finalNotes = [];
    noteGroupings.forEach((notes) => {
        // assumes notes are touching... bad strategy
        const lengthsOfEachNote = notes.map((note) => note.duration);
        const recommendedSubdivisionIndicies = (0, helpers_1.getPartitioningArrayWithMax)(lengthsOfEachNote, maxBreathSeconds);
        finalNotes = [
            ...finalNotes,
            ...(0, helpers_1.sliceAndDice)(notes, recommendedSubdivisionIndicies),
        ];
    });
    return finalNotes;
};
NotesProcessor.clumpTogether = (noteGroupings, maxBreathSeconds) => {
    let finalNotes = [];
    let tempNotes = [];
    noteGroupings.forEach((notes) => {
        const optimisticNotes = [...tempNotes, ...notes];
        if ((0, helpers_1.getDurationOfNotes)(optimisticNotes) <= maxBreathSeconds) {
            tempNotes = [...tempNotes, ...notes];
        }
        else {
            finalNotes.push(tempNotes);
            tempNotes = notes;
        }
    });
    if (tempNotes.length) {
        finalNotes.push(tempNotes);
    }
    return finalNotes;
};
