"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useKeyPress = void 0;
const react_1 = __importStar(require("react"));
const uuid_1 = require("uuid");
const _1 = require(".");
const helpers_1 = require("./helpers");
const midi_1 = __importDefault(require("./midi"));
const X_EXPAND = 200;
function buildNote(note) {
    var _a, _b;
    // NOTE: offset is a unit of time
    const height = 1000 / 127;
    const noteStart = note.time;
    const width = note.duration * X_EXPAND;
    const offset = (_a = note.offset) !== null && _a !== void 0 ? _a : 0;
    const x = noteStart * X_EXPAND + offset * X_EXPAND;
    const y = (127 - note.midi) * height;
    const topLeft = { x, y };
    const bottomRight = { x: x + width, y: y + height };
    return Object.assign(Object.assign({}, note), { x,
        y,
        width,
        height,
        topLeft,
        bottomRight,
        offset, selected: (_b = note.selected) !== null && _b !== void 0 ? _b : false });
}
function getBars(beatLength, width) {
    const res = [];
    let i = 0;
    while (i <= width) {
        res.push(i);
        i += beatLength;
    }
    return res;
}
function renderBars(ctx, bars, height) {
    for (const bar of bars) {
        ctx.beginPath();
        ctx.lineWidth = 1;
        ctx.strokeStyle = "rgba(0, 0, 0, 0.2)";
        const x = bar * X_EXPAND;
        ctx.moveTo(x, 0);
        ctx.lineTo(x, height);
        ctx.stroke();
    }
}
function getRandomColor() {
    const rand = () => Math.floor(Math.random() * 255);
    return `rgb(${rand()}, ${rand()}, ${rand()})`;
}
function renderNote(ctx, note, color) {
    ctx.beginPath();
    ctx.lineWidth = 2;
    ctx.strokeStyle = color !== null && color !== void 0 ? color : "#000000";
    ctx.rect(note.x, note.y, note.width, note.height);
    ctx.stroke();
    if (note.selected) {
        ctx.fillStyle = color !== null && color !== void 0 ? color : "#000000";
        ctx.fill();
    }
    if (note.lyric) {
        ctx.fillStyle = "#000000";
        ctx.font = "30px Arial";
        ctx.fillText(note.lyric, note.x, note.y, note.width);
    }
}
function renderBox(ctx, hi, lo, l, r, col) {
    ctx.beginPath();
    ctx.lineWidth = 1;
    ctx.strokeStyle = col;
    ctx.rect(l, hi, r - l, lo - hi);
    ctx.stroke();
}
function renderSegments(ctx, segments) {
    for (const segment of segments) {
        renderSegment(ctx, segment);
    }
}
function renderSegment(ctx, segment) {
    let hi = Infinity;
    let lo = -Infinity;
    let right = -Infinity;
    for (const note of segment.notes) {
        renderNote(ctx, note, segment.color);
        hi = Math.min(hi, note.topLeft.y);
        lo = Math.max(lo, note.bottomRight.y);
        right = Math.max(right, note.bottomRight.x);
    }
    const left = segment.offset * X_EXPAND;
    renderBox(ctx, hi, lo, left, right, segment.color);
}
function renderNotes(ctx, notes) {
    for (const note of notes) {
        renderNote(ctx, note);
    }
}
function clear(ctx, w, h) {
    ctx.fillStyle = "#ffffff";
    ctx.fillRect(0, 0, w, h);
}
function useKeyPress(targetKey) {
    const [keyPressed, setKeyPressed] = (0, react_1.useState)(false);
    function downHandler(evt) {
        // special case for tab key in this application...
        if (evt.key === "Tab") {
            evt.preventDefault();
        }
        if (evt.key === targetKey) {
            setKeyPressed(true);
        }
    }
    const upHandler = ({ key }) => {
        if (key === targetKey) {
            setKeyPressed(false);
        }
    };
    (0, react_1.useEffect)(() => {
        window.addEventListener("keydown", downHandler);
        window.addEventListener("keyup", upHandler);
        return () => {
            window.removeEventListener("keydown", downHandler);
            window.removeEventListener("keyup", upHandler);
        };
    }, []);
    return keyPressed;
}
exports.useKeyPress = useKeyPress;
function Segmentizer(props) {
    const ref = (0, react_1.useRef)(null);
    const [height] = (0, react_1.useState)(800); // TODO: in the future this could be dynamic?
    const [width, setWidth] = (0, react_1.useState)(2000);
    const [notes, setNotes] = (0, react_1.useState)([]);
    const [addingLyric, setAddingLyric] = (0, react_1.useState)(null);
    const [lyric, setLyric] = (0, react_1.useState)("");
    const [bpm, setBpm] = (0, react_1.useState)();
    const [bars, setBars] = (0, react_1.useState)([]);
    const [segments, setSegments] = (0, react_1.useState)([]);
    const [mouseDown, setMouseDown] = (0, react_1.useState)(null);
    const [beatLength, setBeatLength] = (0, react_1.useState)(null);
    const [shift, setShift] = (0, react_1.useState)(null);
    const escapePressed = useKeyPress("Escape");
    const enterPressed = useKeyPress("Enter");
    const tabPressed = useKeyPress("Tab");
    const tickPressed = useKeyPress("`");
    (0, react_1.useEffect)(() => {
        if (enterPressed) {
            handleSaveLyric();
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [enterPressed]);
    (0, react_1.useEffect)(() => {
        if (tabPressed) {
            const lastNote = handleSaveLyric();
            // find next lyric
            if (lastNote) {
                const possibleNextNotes = notes.filter((n) => n.time > lastNote.time);
                const score = (note) => {
                    const midiDistance = Math.abs(note.midi - lastNote.midi) || 1;
                    const timeDifference = Math.abs(note.time - lastNote.time) * 50;
                    return midiDistance + timeDifference;
                };
                if (possibleNextNotes.length) {
                    const nextNotes = possibleNextNotes.sort((a, b) => score(b) > score(a) ? 1 : -1);
                    setAddingLyric(nextNotes[nextNotes.length - 1].id);
                }
            }
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [tabPressed]);
    (0, react_1.useEffect)(() => {
        if (escapePressed) {
            handleClearSelection();
            setAddingLyric(null);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [escapePressed]);
    (0, react_1.useEffect)(() => {
        if (tickPressed) {
            handleSaveSegment();
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [tickPressed]);
    (0, react_1.useEffect)(() => {
        if (props.data) {
            const buffer = (0, helpers_1.base64ToArrayBuffer)(props.data);
            const midi = midi_1.default.fromBuffer(buffer);
            if (midi) {
                const _beatLength = (0, helpers_1.getBeatLength)(midi.simpleBpm);
                const _shift = _beatLength * 2;
                (0, helpers_1.shiftMidi)(midi, _shift);
                const notes = midi
                    .getAllNotes()
                    .map((n) => (Object.assign(Object.assign({}, n), { id: (0, uuid_1.v4)() })))
                    .map(buildNote)
                    .sort((a, b) => (b.time > a.time ? -1 : 1));
                const maxWidth = Math.max(...notes.map((n) => n.x)) + 300;
                setBpm(midi.simpleBpm);
                setBars(getBars(_beatLength, maxWidth));
                setWidth(maxWidth);
                setBeatLength(_beatLength);
                setShift(_shift);
                setSegments([]);
                setTimeout(() => {
                    // dumb hack to get rid of weird canvas dimension change display issue
                    setNotes(notes);
                }, 100);
            }
        }
    }, [props.data, ref]);
    function handleSaveLyric() {
        const tempNotes = [];
        let res = null;
        if (addingLyric) {
            for (const note of notes) {
                if (note.id === addingLyric) {
                    tempNotes.push(Object.assign(Object.assign({}, note), { lyric: lyric || undefined }));
                    res = note;
                }
                else {
                    tempNotes.push(note);
                }
            }
            setNotes(tempNotes);
            setLyric("");
            setAddingLyric(null);
        }
        return res;
    }
    function handleMouseDown(evt) {
        if (ref.current) {
            const rect = ref.current.getBoundingClientRect();
            const x = evt.clientX - rect.left;
            const y = evt.clientY - rect.top;
            setMouseDown({ x, y });
        }
    }
    function handleClearSelection() {
        const tempNotes = [];
        for (const note of notes) {
            tempNotes.push(note.selected ? Object.assign(Object.assign({}, note), { selected: false, lyric: undefined }) : note);
        }
        setNotes(tempNotes);
    }
    function handleSaveSegment() {
        const remaining = [];
        const segmentNotes = [];
        notes.forEach((note) => {
            if (note.selected) {
                segmentNotes.push(note);
            }
            else {
                remaining.push(note);
            }
        });
        setNotes(remaining);
        if (segmentNotes.length && beatLength && shift) {
            const color = getRandomColor();
            const offset = (0, helpers_1.getSegmentOffset)(segmentNotes[0].time, beatLength, shift);
            const shiftedNotes = segmentNotes.map((n) => 
            // TODO: it's not the most intuitive thing to pass down offset at note level
            buildNote(Object.assign(Object.assign({}, n), { time: n.time - offset, offset })));
            setSegments([
                ...segments,
                { id: (0, uuid_1.v4)(), color, notes: shiftedNotes, offset },
            ]);
        }
    }
    function handleDoubleClick(evt) {
        if (ref.current) {
            const rect = ref.current.getBoundingClientRect();
            const x = evt.clientX - rect.left;
            const y = evt.clientY - rect.top;
            notes.forEach((note) => {
                const { x: noteX, y: noteY, width, height } = note;
                if (x >= noteX &&
                    x <= noteX + width &&
                    y >= noteY &&
                    y <= noteY + height) {
                    setAddingLyric(note.id);
                }
            });
        }
    }
    function handleMouseUp(evt) {
        if (ref.current && mouseDown) {
            const rect = ref.current.getBoundingClientRect();
            const x = evt.clientX - rect.left;
            const y = evt.clientY - rect.top;
            const left = Math.min(x, mouseDown.x);
            const right = Math.max(x, mouseDown.x);
            const top = Math.min(y, mouseDown.y);
            const bottom = Math.max(y, mouseDown.y);
            setMouseDown(null);
            setNotes(notes.map((note) => {
                const { x: noteX, y: noteY, width, height } = note;
                let clicked = false;
                if (noteX + width >= left &&
                    noteX <= right &&
                    noteY + height >= top &&
                    noteY <= bottom) {
                    clicked = true;
                }
                return Object.assign(Object.assign({}, note), { selected: clicked ? !note.selected : note.selected });
            }));
        }
    }
    if (ref.current && notes) {
        const current = ref.current;
        const ctx = current.getContext("2d");
        clear(ctx, width, height);
        renderNotes(ctx, notes);
        renderBars(ctx, bars, height);
        renderSegments(ctx, segments);
    }
    function deleteSegment(segment) {
        if (segments.find((s) => s.id === segment.id)) {
            setSegments(segments.filter((s) => s.id !== segment.id));
            setNotes([
                ...notes,
                ...segment.notes.map((note) => (Object.assign(Object.assign({}, note), { time: note.offset + note.time, selected: false, offset: 0 }))),
            ]);
        }
    }
    function renderButtons() {
        return (react_1.default.createElement("div", { style: { position: "fixed", maxWidth: "800px" } }, segments.map((segment) => (react_1.default.createElement("button", { key: segment.id, onClick: () => deleteSegment(segment), style: {
                backgroundColor: segment.color,
                color: "#FFFFFF",
            } }, "delete")))));
    }
    function handleSave() {
        props.onSave(segments.map((segment) => {
            const notes = [];
            let lowestNote = Infinity;
            let highestNote = -Infinity;
            for (const note of segment.notes) {
                lowestNote = Math.min(lowestNote, note.midi);
                highestNote = Math.max(highestNote, note.midi);
                notes.push({
                    time: note.time,
                    midi: note.midi,
                    duration: note.duration,
                    velocity: 1,
                    lyric: note.lyric,
                });
            }
            return {
                bpm: bpm,
                offset: segment.offset,
                lowestNote,
                highestNote,
                notes,
            };
        }));
    }
    function handleLucky() {
        const segments = (0, _1.segmentizeMidi)(props.data);
        if (!segments) {
            alert("Midi file isn't simple enough to auto-segment...");
            return;
        }
        setNotes([]);
        setSegments(segments.map((segment) => ({
            id: (0, uuid_1.v4)(),
            color: getRandomColor(),
            offset: segment.offset,
            notes: segment.notes
                .map((note) => (Object.assign(Object.assign({}, note), { offset: segment.offset })))
                .map((n) => (Object.assign(Object.assign({}, n), { id: (0, uuid_1.v4)() })))
                .map(buildNote),
        })));
    }
    return (react_1.default.createElement("div", null,
        react_1.default.createElement("div", { style: { position: "fixed" } },
            react_1.default.createElement("button", { onClick: handleSaveSegment }, "make segment"),
            react_1.default.createElement("button", { onClick: handleLucky }, "i'm feeling lucky"),
            react_1.default.createElement("button", { disabled: !segments.length, onClick: handleSave }, "save segments"),
            addingLyric ? (react_1.default.createElement("input", { autoFocus: true, value: lyric, onChange: (e) => setLyric(e.target.value), tabIndex: 1 })) : null),
        react_1.default.createElement("canvas", { onMouseDown: handleMouseDown, onMouseUp: handleMouseUp, onDoubleClick: handleDoubleClick, id: "react-midiVisualizer-canvas", ref: ref, height: height, width: width }),
        renderButtons()));
}
exports.default = Segmentizer;
